/** 
* Expands exponential notation to decimal notation
*/
String.prototype.expandExponential = function(){//v1.0
    return this.replace(/^([+-])?(\d+).?(\d*)[eE]([-+]?\d+)$/, function(x, s, n, f, c){
        var l = +c < 0, i = n.length + +c, x = (l ? n : f).length,
        c = ((c = Math.abs(c)) >= x ? c - x + l : 0),
        z = (new Array(c + 1)).join("0"), r = n + f;
        return s + (l ? r = z + r : r += z).substr(0, i += l ? z.length : 0) + (i < r.length ? "." + r.substr(i) : "");
    });
};

/**
* Removes white spaces from the end/beginning of a string
*/
function trim (str) {
	
	return str.replace(/^\s+|\s+$/g, '');
					   
}

/**
* Displays the number with a fixed number of characters
*/
function formatNumber (tempNum) {
	
	var fmtNum;
	var strNum = tempNum.toString();
	
	// formats number in exponential notation ie: -1e-8 or 1e8
	if (strNum.indexOf('e') > -1) strNum = strNum.expandExponential();
	
	// for numbers longer than the MAX_LENGTH 
	if (strNum.length > MAX_LENGTH) {
	
		// verifies if number has decimal point
		if  (strNum.indexOf('.') > -1) {
		
		    // separates the integer part from decimal part
		    var strIntPart = strNum.substr(0, strNum.indexOf('.'));
		    
		    if (strIntPart.length <= MAX_LENGTH -2) {
		    	// treatment for numbers with at least one decimal place ie:12345678.0  maxlen=10
		     	strNum = Number(strNum.substr(0,MAX_LENGTH)).toString();
		     	
		     	// in some cases the result of Number() conversion above results on exponential notation ex 3e-8
		     	if (strNum.indexOf('e') > -1) strNum = strNum.expandExponential();
		     	
		     	// treatment for numbers like 0.000000000247105381467165 that would result in 0.000000000 
		     	if (strNum == 0) strNum = ERR_RESULT_TOOLONG;
		     	
		    }
		    else if ((strIntPart.length > MAX_LENGTH -2) && (strIntPart.length <= MAX_LENGTH)) { 	
		     	// round numbers that will loose the decimal part to fit length ie: 1234567890.0  maxlen=10
		     	strNum = Number(Math.round(tempNum)).toString();
		     	 
			}    
			else {
		    	// numbers in witch the integer part exceeds the max length  
		    	strNum = ERR_RESULT_TOOLONG;
		    }	
		}	
		
		// numbers with no decimal part and in witch the integer part exceeds the max length   
		else { 
		
			strNum = ERR_RESULT_TOOLONG;
		}	
	}	
	
	return strNum;		
}

/**
* Numeric validation 
*/
function isValidDec (strNum) {
	
	var re_decNum = /^[-+]?\d+(\.\d+)?$/;

	if (!re_decNum.exec(strNum)) return false;
	
	return true;	
}

function isValidInt (strNum) {
	
	var re_intNum = /^-?\d+$/;

	if (!re_intNum.exec(strNum)) return false;
	
	return true;	
}

function isValidBin (strBin) {

	var re_binNum = /^[0-1]+$/;
	
	if (!re_binNum.exec(strBin)) return false;
	
    return true;
}       

function isValidOct (strOct) {

	var re_octNum = /^[0-7]+$/;
	
	if (!re_octNum.exec(strOct)) return false;
	
    return true;
}       

function isValidHex (strHex) {

	var re_hexNum = /^[\da-f]+$/i;

	if (!re_hexNum.exec(strHex)) return false;
	
	return true;	
}	

function isNumberBaseValid (tempConvertFromType, strNumber) {

	var bValidation = true ;
	
	if ((tempConvertFromType == TYPE_NC_DEC && !isValidDec(strNumber)) ||   
	    (tempConvertFromType == TYPE_NC_BIN && !isValidBin(strNumber)) || 
        (tempConvertFromType == TYPE_NC_OCT && !isValidOct(strNumber)) || 
	    (tempConvertFromType == TYPE_NC_HEX && !isValidHex(strNumber)) || 
	    (parseInt(strNumber) < 0)) {
	   
	   bValidation = false;
	}   
	
	return bValidation;
}

 

/**
* Numeric base conversion formula  		 			 
*/
function convertNumberBase (tempConvertFromType, tempConvertToType, tempInputValue) {
	
	var tempOutputValue;
	var tempDec = 0;
	
	if (isNumberBaseValid(tempConvertFromType, tempInputValue)) {
	
		// Converts all types to decimal 
		if (tempConvertFromType == TYPE_NC_DEC) tempDec = parseInt(tempInputValue);
		else if (tempConvertFromType == TYPE_NC_BIN) tempDec = parseInt(tempInputValue,2);
	    else if (tempConvertFromType == TYPE_NC_OCT) tempDec = parseInt(tempInputValue,8);
		else if (tempConvertFromType == TYPE_NC_HEX) tempDec = parseInt(tempInputValue,16);
		
		//println ('Converted number: ' + tempDec); 
		
		// Converts from decimal to all other types 
		if (tempConvertToType == TYPE_NC_DEC)	   tempOutputValue = tempDec; 
		else if (tempConvertToType == TYPE_NC_BIN) tempOutputValue = tempDec.toString(2); 
		else if (tempConvertToType == TYPE_NC_OCT) tempOutputValue = tempDec.toString(8); 
		else if (tempConvertToType == TYPE_NC_HEX) tempOutputValue = tempDec.toString(16).toUpperCase();
		isValidInput =  true;
		
		if (tempOutputValue.length > MAX_LENGTH) {
			
			tempOutputValue = ERR_RESULT_TOOLONG;
		}
	}	
	else {
	
		tempOutputValue = ERR_INVALID_INPUT;
		isValidInput =  false;
	}
	
	return tempOutputValue;
}


/**
* Temperature conversion formula  		 			 
*/
function convertTemperature (strFromType, strToType, strInputValue) {

    var tempOutputValue;
    
    if (!isNaN(strInputValue) && isValidDec(strInputValue)) {
    
    	var floatInputValue = parseFloat(eval(strInputValue));
    	
	    if (strFromType == strToType) tempOutputValue = Number(strInputValue).toString();
	    else if (strFromType == 'Celsius'   && strToType == 'Farenheit') tempOutputValue = 9 * floatInputValue / 5 + 32;
	    else if (strFromType == 'Celsius'   && strToType == 'Kelvin')    tempOutputValue = floatInputValue + 273.15;
	    else if (strFromType == 'Farenheit' && strToType == 'Celsius')   tempOutputValue = 5 * (floatInputValue - 32) / 9;
	    else if (strFromType == 'Farenheit' && strToType == 'Kelvin')    tempOutputValue = 5 * (floatInputValue - 32) / 9 + 273.15;
	    else if (strFromType == 'Kelvin'    && strToType == 'Celsius')   tempOutputValue = floatInputValue - 273.15;
	    else if (strFromType == 'Kelvin'    && strToType == 'Farenheit') tempOutputValue = 9 * (floatInputValue - 273.15) / 5 + 32;
	    
	    //println ( 'result:' + tempOutputValue );
	    tempOutputValue = formatNumber(tempOutputValue);
	    isValidInput = true;
	}
	else {
	
		tempOutputValue = ERR_INVALID_INPUT;
		isValidInput = false;
	}
	
	return tempOutputValue;
}

/**
* Conversion formula for all other types of conversion 		 			 
*/
function commonConverter (strFromValue, strToValue, strInputValue) { 

	var tempOutputValue;
    var floatFromValue  = parseFloat(eval(strFromValue));
    var floatToValue    = parseFloat(eval(strToValue));

	if (!isNaN(strInputValue) && isValidDec(strInputValue)) {
	
		if (strFromValue == strToValue) { 
			// when no conversion is needed only formats number correctly and display (ie: -0.00000 -> 0)
			// this is done because after multiplying and dividing, number precision may be different from the
			// value initially entered   
			tempOutputValue = Number(strInputValue).toString(); 
			
		}
		else {

		    // calculates the conversion 
	    	var floatInputValue = parseFloat(strInputValue);
		    tempOutputValue = floatInputValue *  floatToValue / floatFromValue;
		}
		    
	    //println ( 'result:' + tempOutputValue );
	    tempOutputValue = formatNumber(tempOutputValue);
	    isValidInput = true;
	}
	else {
	
		tempOutputValue = ERR_INVALID_INPUT;
	    isValidInput = false;
	}
	
	return tempOutputValue;
}	        
